// Index.cpp : implementation file
//

#include "stdafx.h"
#include "PCI Explorer.h"
#include "pci.h"
#include "pciScanIoctl.h"

#include "NumericEdit.h"
#include "IDCombo.h"
#include "DataDisplay.h"
#include "pcidata.h"

#include "Index.h"

#include "codes.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CIndex property page

IMPLEMENT_DYNCREATE(CIndex, CPropertyPage)

CIndex::CIndex() : CPropertyPage(CIndex::IDD)
    {
     //{{AFX_DATA_INIT(CIndex)
     // NOTE: the ClassWizard will add member initialization here
     //}}AFX_DATA_INIT
     CString s;
     s.LoadString(IDS_INDEX);

     TCHAR * title = new TCHAR[s.GetLength() + 1];
     lstrcpy(title, s);

     m_psp.pszTitle = title;
     m_psp.dwFlags |= PSP_USETITLE;
}

CIndex::~CIndex()
{
}

void CIndex::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CIndex)
	DDX_Control(pDX, IDC_GOTO, c_Goto);
	DDX_Control(pDX, IDC_INDEX, c_Index);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CIndex, CPropertyPage)
	//{{AFX_MSG_MAP(CIndex)
	ON_BN_CLICKED(IDC_GOTO, OnGoto)
	ON_NOTIFY(TVN_SELCHANGED, IDC_INDEX, OnSelchangedIndex)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CIndex message handlers

/****************************************************************************
*                              CIndex::addVendor
* Inputs:
*       CString & shortVendor
*	CString & longVendor
*	WORD VendorID
* Result: HTREEITEM
*       The vendor item
* Effect: 
*       
****************************************************************************/

HTREEITEM CIndex::addVendor(CString & shortVendor, CString & longVendor, WORD VendorID)
    {
     for(HTREEITEM item = c_Index.GetNextItem(NULL, TVGN_CHILD);
         item != NULL;
	 item = c_Index.GetNextItem(item, TVGN_NEXT))
         { /* scan tree */
	  CString key = c_Index.GetItemText(item);
	  if(lstrcmp(key, shortVendor) == 0)
	     { /* found vendor */
	      return item;
	     } /* found vendor */
	 } /* scan tree */
      
      // It wasn't in the tree. Add it
      item = c_Index.InsertItem(shortVendor);

      CString id;
      id.Format(_T("0x%04x"), VendorID);
      c_Index.InsertItem(id, item);
      c_Index.InsertItem(longVendor, item);

      return item;
    }

/****************************************************************************
*                              CIndex::addDevice
* Inputs:
*       HTREEITEM vendor: Vendor entry
*	PCIData * data: Data page
* Result: void
*       
* Effect: 
*       Adds the device description
****************************************************************************/

void CIndex::addDevice(HTREEITEM vendor, PCIData * data, int i)
    {
     CString chip;
     CString desc;
     CString deviceName;
     PCISCAN_OUTPUT * pci = &data->data;

     if(findDevice(pci->PciData.VendorID, pci->PciData.DeviceID, chip, desc))
        { /* found it */
	 if(lstrlen(chip) == 0 && lstrlen(desc) != 0)
	    { /* I */
	     deviceName = desc;
	    } /* I */
	 else
	 if(lstrlen(chip) != 0 && lstrlen(desc) == 0)
	    { /* II */
	     deviceName = chip;
	    } /* II */
	 else
	 if(lstrlen(chip) != 0 && lstrlen(desc) != 0)
	    { /* III */
	     deviceName = desc;
	    } /* III */
	 else
	 if(lstrlen(chip) == 0 && lstrlen(desc) == 0)
	    { /* IV */
	     deviceName.Format(_T("%04x"), pci->PciData.DeviceID);
	    } /* IV */
	} /* found it */

     HTREEITEM device = c_Index.InsertItem(deviceName, vendor);

     c_Index.SetItemData(device, (DWORD)i);

     CString fmt;
     fmt.LoadString(IDS_BUSDEVFCN);
     CString s;
     s.Format(fmt, pci->BusNumber, 
     		   pci->SlotData.u.bits.DeviceNumber,
		   pci->SlotData.u.bits.FunctionNumber);

     c_Index.InsertItem(s, device);

     fmt.LoadString(IDS_DEVICEID);
     s.Format(fmt, pci->PciData.DeviceID);

     c_Index.InsertItem(s, device);

     if(chip.GetLength() > 0 && deviceName != chip)
        { /* chip */
	 fmt.LoadString(IDS_CHIPID);
	 s.Format(fmt, chip);
	 c_Index.InsertItem(s, device);
	} /* chip */

     if(desc.GetLength() > 0 && deviceName != desc)
	c_Index.InsertItem(desc, device);
    }

BOOL CIndex::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	
 	// [-] Vendor
 	//  |  |
	//  |  +-- id
	//  |  |
	//  |  +-- name
	//  |  |
	//  | [-]-- Device
	//  |  |    |
	//  |  |    +-- bus:dev:fcn
	//  |  |    |
	//  |  |    +-- id
	//  |  |    |
	//  |  |    +-- chip
	//  |  |    |
	//  |  |    +-- desc
	//  |  |
	//  | [+]-- Device
	//  |  |
	//  | [+]-- Device
	//  |
        // [-] Vendor
	//  |
	// [-] Vendor


	CPropertySheet * sheet = (CPropertySheet *)GetParent();

	int count = sheet->GetPageCount();
	for(int i = 0; i < count; i++)
	   { /* scan pages */
	    CWnd * page = sheet->GetPage(i);

	    if(page->IsKindOf(RUNTIME_CLASS(PCIData)))
	       { /* PCI data */
		PCIData * data = (PCIData *)page;
		CString shortVendor;
		CString longVendor;
		if(!findVendor(data->data.PciData.VendorID, 
		   				shortVendor, longVendor))
		   { /* failed */
		    shortVendor.LoadString(IDS_UNKNOWN);
		    longVendor = _T("");
		   } /* failed */
		HTREEITEM vendor = addVendor(shortVendor, longVendor, 
					      data->data.PciData.VendorID);
		
                addDevice(vendor, data, i);
	       } /* PCI data */
	   } /* scan pages */
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CIndex::OnGoto() 
    {
     HTREEITEM item = c_Index.GetSelectedItem();

     if(item != NULL)
        { /* scan */
	 while(TRUE)
	    { /* scan upwards */
	     DWORD data = c_Index.GetItemData(item);
	     if(data != NULL)
	        { /* found it */
		 CPropertySheet * sheet = (CPropertySheet *)GetParent();
		 sheet->SetActivePage((int)data);
		 break;
		} /* found it */
	     item = c_Index.GetParentItem(item);
	     if(item == NULL)
		break;
	    } /* scan updwards */
	} /* scan */
}

/****************************************************************************
*                           CIndex::updateControls
* Result: void
*       
* Effect: 
*       Updates the controls based on the current state
****************************************************************************/

void CIndex::updateControls()
    {
     HTREEITEM item = c_Index.GetSelectedItem();

     BOOL found = FALSE;

     if(item != NULL)
        { /* scan */
	 while(TRUE)
	    { /* scan upwards */
	     DWORD data = c_Index.GetItemData(item);
	     if(data != NULL)
	        { /* found it */
		 c_Goto.EnableWindow(TRUE);
		 found = TRUE;
		 break;
		} /* found it */
	     item = c_Index.GetParentItem(item);
	     if(item == NULL)
		break;
	    } /* scan upwards */
	} /* scan */

     if(!found)
	c_Goto.EnableWindow(FALSE);
    }


void CIndex::OnSelchangedIndex(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	
	updateControls();
	
	*pResult = 0;
}
