// PCIStartup.cpp : implementation file
//

#include "stdafx.h"
#include <winioctl.h>
#include <winsvc.h> // cleverly excluded by Microsoft

#include "PCI Explorer.h"
#include "PCIStartup.h"
#include "pci.h"
#include "pciScanIoctl.h"
#include "about.h"

#include "GetDriveType.h"

#include "PCI ExplorerDlg.h"
#include "NumericEdit.h"
#include "IDCombo.h"
#include "DataDisplay.h"
#include "pcidata.h"
#include "RegVars.h"
#include "index.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define PCISCAN _T("PCISCAN")

static WORD UWM_RUN = ::RegisterWindowMessage(_T("UWM_RUN-{E7844BB0-105D-11d2-8379-00AA005C0507}"));

/////////////////////////////////////////////////////////////////////////////
// CPCIStartup dialog


CPCIStartup::CPCIStartup(CWnd* pParent /*=NULL*/)
	: CDialog(CPCIStartup::IDD, pParent)
{
	//{{AFX_DATA_INIT(CPCIStartup)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CPCIStartup::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPCIStartup)
	DDX_Control(pDX, IDC_DRIVER_CAPTION, c_c_Driver);
	DDX_Control(pDX, IDC_DRIVER, c_Driver);
	DDX_Control(pDX, IDC_CHANGE, c_Change);
	DDX_Control(pDX, IDC_BROWSE, c_Browse);
	DDX_Control(pDX, IDC_COMPLETED, c_Completed);
	DDX_Control(pDX, IDC_OPENING, c_Opening);
	DDX_Control(pDX, IDC_PROGRESS, c_Progress);
	DDX_Control(pDX, IDOK, c_OK);
	DDX_Control(pDX, IDC_UNINSTALLING, c_Uninstalling);
	DDX_Control(pDX, IDC_STOPPING, c_Stopping);
	DDX_Control(pDX, IDC_STATUS, c_Status);
	DDX_Control(pDX, IDC_STARTING, c_Starting);
	DDX_Control(pDX, IDC_SCM_OPEN, c_OpeningSCM);
	DDX_Control(pDX, IDC_SCANNING, c_Scanning);
	DDX_Control(pDX, IDC_INSTALLING, c_Installing);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPCIStartup, CDialog)
	//{{AFX_MSG_MAP(CPCIStartup)
	ON_WM_SYSCOMMAND()
	ON_REGISTERED_MESSAGE(UWM_RUN, OnRunPCI)
	ON_BN_CLICKED(IDC_BROWSE, OnBrowse)
	ON_BN_CLICKED(IDC_CHANGE, OnChange)
	ON_EN_CHANGE(IDC_DRIVER, OnChangeDriver)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/****************************************************************************
*                       CPCIStartup::InstallDriver
* Inputs:
*       IN SC_HANDLE scm: Service Control Manager handle
*	IN LPCTSTR DriverName: Driver name
*	IN LPCTSTR executable: Place where it is found
* Result: BOOL
*       TRUE if successful
*	FALSE if error
* Effect: 
*       Installs the driver
****************************************************************************/

BOOL CPCIStartup::InstallDriver(SC_HANDLE  scm, LPCTSTR DriverName, LPCTSTR ServiceExe)
    {
     SC_HANDLE  Service;
     DWORD      err;

     //
     // NOTE: This creates an entry for a standalone driver. If this
     //       is modified for use with a driver that requires a Tag,
     //       Group, and/or Dependencies, it may be necessary to
     //       query the registry for existing driver information
     //       (in order to determine a unique Tag, etc.).
     //

     Service = CreateService (scm,          // SCManager database
     			      DriverName,           // name of service
			      DriverName,           // name to display
			      SERVICE_ALL_ACCESS,    // desired access
			      SERVICE_KERNEL_DRIVER, // service type
			      SERVICE_DEMAND_START,  // start type
			      SERVICE_ERROR_NORMAL,  // error control type
			      ServiceExe,            // full path to service's binary
			      NULL,                  // no load ordering group
			      NULL,                  // no tag identifier
			      NULL,                  // no dependencies
			      NULL,                  // LocalSystem account
			      NULL                   // no password
			      );

     if (Service == NULL)
	{ /* failed */
	 err = GetLastError();

	 if (err == ERROR_SERVICE_EXISTS)
	    { /* already installed */
	    } /* already installed */
	 else
	    {
	     printf ("failure: CreateService (0x%02x)\n",
	     err
	     );
	    }

	 return FALSE;
	}
     else
	{
	 printf ("CreateService SUCCESS\n");
	}

     CloseServiceHandle (Service);

     return TRUE;
    }

/****************************************************************************
*                        CPCIStartup::RemoveDriver
* Inputs:
*       SC_HANDLE scm: Service control manager handle
* Result: BOOL
*       TRUE if successful
*	FALSE if failure
* Effect: 
*       Uninstalles the PCI driver
****************************************************************************/

BOOL CPCIStartup::RemoveDriver(SC_HANDLE scm, LPCTSTR DriverName)
    {
     SC_HANDLE  Service;
     BOOL       ret;

     Service = OpenService (scm,
                            DriverName,
			    SERVICE_ALL_ACCESS
			    );

     if (Service == NULL)
	{
	 return FALSE;
	}

     ret = DeleteService (Service);

     if (!ret)
	{ /* failed to remove it */
	} /* failed to remove it */

     CloseServiceHandle (Service);

     return ret;
    }


/****************************************************************************
*                        CPCIStartup::getErrorMessage
* Inputs:
*       DWORD err: Error code to be displayed
* Result: CString
*       Displayable string with superfluous \r\n removed
* Effect: 
*       
****************************************************************************/

CString CPCIStartup::getErrorMessage(DWORD err)
    {
     CString s;
     LPTSTR msg;

     if(::FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM |
		  FORMAT_MESSAGE_ALLOCATE_BUFFER,
		  NULL,
		  err,
		  0,
		  (LPTSTR)&msg,
		  0,
		  NULL) == 0)
	 { /* failed */
	  CString fmt;
	  fmt.LoadString(IDS_FORMAT_ERROR);
	  s.Format(fmt, err, err);
	 } /* failed */
      else
	 { /* success */
	  s = msg;

	  int n;
	  for(n = s.FindOneOf(_T("\r\n")); n >= 0; n = s.FindOneOf(_T("\r\n")))
	     { /* delete internal */
	      if(s[n] == _T('\n'))
	          s = s.Left(n) + _T(" ") + s.Mid(n+1);
	      else
		  s = s.Left(n) + s.Mid(n+1);
	     } /* delete internal */
	  LocalFree(msg);
	 } /* success */
      return s;
    }

/****************************************************************************
*                           CPCIStartup::showError
* Inputs:
*       DWORD err: Error code to display
*	UINT msg: Error message to display
*	CString & other: Additional information
* Result: void
*       
* Effect: 
*       Displays the error in the status box
****************************************************************************/

void CPCIStartup::showError(DWORD err, UINT msg, CString & other)
    {
     CString s;
     s.LoadString(msg);
     s += _T("\r\n");
     s += getErrorMessage(err);
     if(other.GetLength() > 0)
        { /* add other */
	 s += _T("\r\n");
	 s += other;
	} /* add other */

     c_Status.SetWindowText(s);
    }

/****************************************************************************
*                        CPCIStartup::StartDriver
* Inputs:
*       SC_HANDLE scm: Handle to SCM
*	LPCTSTR DriverName: Name of driver to start
* Result: BOOL
*       TRUE if successful
*	FALSE if error
* Effect: 
*       Starts the driver
****************************************************************************/

BOOL CPCIStartup::StartDriver(SC_HANDLE  scm, LPCTSTR DriverName)
    {
     SC_HANDLE  Service;
     BOOL       ret;
     DWORD      err;

     Service = OpenService (scm, DriverName, SERVICE_ALL_ACCESS);

    if (Service == NULL)
       { /* open failed */
        return FALSE;
       } /* open failed */

    ret = StartService (Service,    // service identifier
                        0,             // number of arguments
                        NULL           // pointer to arguments
                        );
    if (!ret)
       { /* failed */
	err = GetLastError();

	if (err == ERROR_SERVICE_ALREADY_RUNNING)
	   {
	    ret = TRUE; // OK, it is already up!
	   }
        else
	   { /* other failure */
	   } /* other failure */
       } /* failed */

    CloseServiceHandle (Service);

    return ret;
}

/****************************************************************************
*                         CPCIStartup::StopDriver
* Inputs:
*       SC_HANDLE scm:
*	LPCTSTR DriverName: driver
* Result: BOOL
*       TRUE if successful
*	FALSE if error
* Effect: 
*       Stops the driver
****************************************************************************/

BOOL CPCIStartup::StopDriver(SC_HANDLE  scm, LPCTSTR DriverName)
    {
     SC_HANDLE       Service;
     BOOL            ret;
     SERVICE_STATUS  serviceStatus;

     Service = OpenService (scm, DriverName, SERVICE_ALL_ACCESS );

     if (Service == NULL)
	{ /* can't stop */
         DWORD err = GetLastError();
	 return FALSE;
	} /* can't stop */

     ret = ControlService (Service, SERVICE_CONTROL_STOP, &serviceStatus);

     if (!ret)
	{
	 DWORD err = GetLastError();
	}

     CloseServiceHandle (Service);
     
     return ret;
    }


/////////////////////////////////////////////////////////////////////////////
// CPCIStartup message handlers

void CPCIStartup::OnOK() 
{
	// TODO: Add extra validation here
	
	CDialog::OnOK();
}

void CPCIStartup::OnCancel() 
{
	// TODO: Add extra cleanup here
	
	CDialog::OnCancel();
}

#define STATE_STARTING_SCM 0
#define STATE_INSTALLING   1
#define STATE_STARTING     2
#define STATE_OPENING	   3
#define STATE_SCANNING     4
#define STATE_WATCHING     5
#define STATE_CLOSING      6
#define STATE_STOPPING     7
#define STATE_REMOVING     8
#define STATE_COMPLETED    9
#define STATE_ERROR       10 

/****************************************************************************
*                        CPCIStartup::disableControls
* Inputs:
*       UINT state: State at which error occurred
* Result: void
*       
* Effect: 
*       Disables all the remaining controls
****************************************************************************/

void CPCIStartup::disableControls(UINT state)
    {
     switch(state)
        { /* state */
	 case STATE_STARTING_SCM:
		 c_Installing.EnableWindow(FALSE);
		 c_Starting.EnableWindow(FALSE);
		 c_Opening.EnableWindow(FALSE);
		 c_Scanning.EnableWindow(FALSE);
		 c_Stopping.EnableWindow(FALSE);
		 c_Uninstalling.EnableWindow(FALSE);
		 break;
	 case STATE_INSTALLING:
		 c_Starting.EnableWindow(FALSE);
		 c_Opening.EnableWindow(FALSE);
		 c_Scanning.EnableWindow(FALSE);
		 c_Stopping.EnableWindow(FALSE);
		 c_Uninstalling.EnableWindow(FALSE);
		 break;
	 case STATE_STARTING:
		 c_Opening.EnableWindow(FALSE);
		 c_Scanning.EnableWindow(FALSE);
		 c_Stopping.EnableWindow(FALSE);
		 break;
	 case STATE_OPENING:
		 c_Scanning.EnableWindow(FALSE);
		 break;
	 case STATE_SCANNING:
		 break;
	 case STATE_WATCHING:
		 break;
	 case STATE_STOPPING:
		 c_Uninstalling.EnableWindow(FALSE);
		 break;
	 case STATE_REMOVING:
		 break;
	} /* state */
    }


/****************************************************************************
*			 CPCIStartup::isValidFileName
* Inputs:
*       CString & filename: Name of file
*	LPUINT msg: Place to put reason for failure, or NULL
* Result: BOOL
*       TRUE if file exists and is on the local machine
*	FALSE if the file does not exist or is not on the local machine
****************************************************************************/

BOOL CPCIStartup::isValidFileName(CString & filename, UINT * msg /* = NULL */)
    {
     HANDLE h = ::CreateFile(filename, 
			     GENERIC_READ,
			     0, 
			     NULL,
			     OPEN_EXISTING,
			     FILE_ATTRIBUTE_NORMAL,
			     NULL);
     if(h != INVALID_HANDLE_VALUE)
	{ /* exists */
	 ::CloseHandle(h);

	 UINT type = getDriveType(filename);
	 switch(type)
	    { /* type */
	     case DRIVE_REMOVABLE:
		     if(msg != NULL)
			*msg = IDS_REMOVABLE_DRIVE;
		     return FALSE;
	     case DRIVE_REMOTE:
		     if(msg != NULL)
			*msg = IDS_NETWORK_DRIVE;
		     return FALSE;
             case DRIVE_FIXED:
		     break; // fixed drive is OK
             case DRIVE_CDROM:
		     if(msg != NULL)
			*msg = IDS_CDROM_DRIVE;
                     return FALSE;
	     case DRIVE_NO_ROOT_DIR:
		     if(msg != NULL)
			 *msg = IDS_NO_ROOT_DIR;
		     return FALSE; 
             default:
		     if(msg != NULL)
			*msg = IDS_INVALID_DRIVE;
                     return FALSE;
	    } /* type */

	 // Now check to see if the file is executable
	 DWORD exetype;
	 BOOL result = GetBinaryType(filename, &exetype);
	 if(!result || exetype != SCS_32BIT_BINARY)
	    { /* not executable */
	     if(msg != NULL)
		*msg = IDS_NOT_EXECUTABLE;
	     return FALSE;
	    } /* not executable */

	 return TRUE;
	} /* exists */
     
     if(msg != NULL)
	*msg = IDS_NO_FILE;
     return FALSE;
    }
	

/****************************************************************************
*                             CPCIStartup::browse
* Inputs:
*       CString & filename: Current best guess filename
* Result: BOOL
*       TRUE if filename supplied
*	FALSE if filename not supplied (Cancel)
* Effect: 
*       Prompts user for file name
****************************************************************************/

BOOL CPCIStartup::browse(CString & filename)
    {
     if(filename.GetLength() == 0)
	filename = PCISCAN _T(".sys");

     CString sysfilter;
     sysfilter.LoadString(IDS_SYS_FILTER);

     CFileDialog dlg(TRUE, _T(".sys"), 
		     filename,
		     OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		     sysfilter);

     CString title;
     title.LoadString(IDS_SYS_TITLE);
     dlg.m_ofn.lpstrTitle = title;


     switch(dlg.DoModal())
	{ /* result */
	 case 0:
	 case IDCANCEL:
		 ::SetLastError(ERROR_FILE_NOT_FOUND);
		 return FALSE;
	 case IDOK:
		 filename = dlg.GetPathName();
		 return TRUE;
	} /* result */

     return FALSE; // impossible, but necessary for compiler happiness
    }

/****************************************************************************
*                         CPCIStartup::checkExistence
* Inputs:
*       CString & filename: File name to check for
* Result: BOOL
*       TRUE if file name exists, or has been changed to the name of
*	an existing file name
*	FALSE if the file name does not exist and the user cancelled
* Effect: 
*       Prompts the user for the location of the driver
****************************************************************************/

BOOL CPCIStartup::checkExistence(CString & filename)
    {
     while(TRUE)
        { /* loop */
	 if(isValidFileName(filename))
	    return TRUE;
	 
         if(!browse(filename))
	    return FALSE;
	} /* loop */
    }

/****************************************************************************
*                         CPCIStartup::updateControls
* Inputs:
*	UINT * msg: Pointer to return error code, or NULL
* Result: void
*       
* Effect: 
*       Updates the controls
****************************************************************************/

void CPCIStartup::updateControls(UINT * msg)
    {
     c_c_Driver.EnableWindow(!running);
     c_Driver.EnableWindow(!running);
     c_Browse.EnableWindow(!running);
     BOOL enable = !running;
     enable &= c_Driver.GetWindowTextLength() > 0;
     if(msg != NULL)
	*msg = 0;
     if(enable)
        { /* see if valid name */
	 CString filename;
	 c_Driver.GetWindowText(filename);
	 enable &= isValidFileName(filename, msg);
	} /* see if valid name */

     c_Change.EnableWindow(enable);
    }

/****************************************************************************
*			     CPCIStartup::OnRunPCI
* Inputs:
*       WPARAM: unused
*	LPARAM: unused
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Runs the PCI startup/shutdown sequence
****************************************************************************/

LRESULT CPCIStartup::OnRunPCI(WPARAM, LPARAM)
    {
     CIndex index;
     UINT state = STATE_STARTING_SCM;
     SC_HANDLE scm = NULL;
     CString additional;
     UINT errorstate;
     UINT errormessage;
     DWORD err = 0;
     UINT stateRecover = STATE_COMPLETED;
     HANDLE pci;
     RegistryString filename(IDS_PCISCAN);
     CPCIExplorerDlg dlg(IDS_CAPTION);

     running = TRUE;

     updateControls();

     // force window to display
     ShowWindow(SW_SHOW);
     UpdateWindow(); 

     while(running)
	{ /* run loop */
	 switch(state)
	    { /* state */
	     case STATE_ERROR:
		     showError(err, errormessage, additional);
		     disableControls(errorstate);
		     state = stateRecover;
		     continue;

	     case STATE_STARTING_SCM:
		     c_OpeningSCM.SetCheck(BST_CHECKED);

		     scm = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);

		     if(scm == NULL)
			{ /* failed */
			 err = GetLastError();
			 errorstate = state;
			 errormessage = IDS_SCM_OPEN_FAILURE;
			 state = STATE_ERROR;
			 continue;
			} /* failed */

		     state = STATE_INSTALLING;
		     break;

	     case STATE_INSTALLING:
		     c_Installing.SetCheck(BST_CHECKED);
		     
		     filename.load(PCISCAN _T(".sys"));
		     if(!checkExistence(filename.value))
			{ /* does not exist */
			 err = GetLastError();
			 errorstate = state;
			 errormessage = IDS_ERROR_INSTALLING;
			 state = STATE_ERROR;
			 continue;
			} /* does not exist */

		     if(!InstallDriver(scm, PCISCAN, filename.value))
			{ /* install failed */
			 err = GetLastError();
			 errorstate = state;
			 errormessage = IDS_ERROR_INSTALLING;
			 additional = filename.value;
			 state = STATE_ERROR;
			 continue;
			} /* install failed */

		     state = STATE_STARTING;
		     stateRecover = STATE_REMOVING;
		     filename.store();
		     c_Driver.SetWindowText(filename.value);
		     break;
	     case STATE_STARTING:
		     c_Starting.SetCheck(BST_CHECKED);
		     if(!StartDriver(scm, PCISCAN))
			{ /* start failed */
			 err = GetLastError();
			 errorstate = state;
			 errormessage = IDS_ERROR_STARTING;
			 state = STATE_ERROR;
			 continue;
			} /* start failed */

		     state = STATE_OPENING;
		     stateRecover = STATE_STOPPING;
		     ::CloseServiceHandle(scm);
		     scm = NULL;
		     break;
	     case STATE_OPENING:
		     c_Opening.SetCheck(BST_CHECKED);
		     pci = ::CreateFile(_T("\\\\.\\") PCISCAN,
						  GENERIC_READ,
						  0,
						  NULL,
						  OPEN_EXISTING,
						  FILE_ATTRIBUTE_NORMAL,
						  NULL);

		     if(pci == INVALID_HANDLE_VALUE)
			{ /* failed */
			 err = GetLastError();
			 errorstate = state;
			 errormessage = IDS_ERROR_OPENING;
			 state = STATE_ERROR;
			 continue;
			} /* failed */

		     state = STATE_SCANNING;
		     break;
	     case STATE_SCANNING:
		     { /* do enumeration */
		      PCISCAN_OUTPUT pciBuffer;
		      c_Scanning.SetCheck(BST_CHECKED);
		      c_Progress.SetRange(0, 3); // may loop several times
		      c_Progress.ShowWindow(SW_SHOW);
		      BOOL result;
		      DWORD length;
		      
		      c_Progress.StepIt();
		      ZeroMemory(pciBuffer.PciData.DeviceSpecific,
		      		 sizeof(pciBuffer.PciData.DeviceSpecific));

		      result = DeviceIoControl( pci,
		      				IOCTL_PCISCAN_GET_FIRST,
						NULL, 0, // no input
						&pciBuffer,
						sizeof(pciBuffer),
						&length,
						NULL);
                      if(!result)
		         { /* failed */
			  // perhaps no PCI bus
			  err = GetLastError();
			  errormessage = IDS_PCISCAN_FAILED;
			  errorstate = state;
			  state = STATE_ERROR;
			  stateRecover = STATE_CLOSING;
			  continue;
			 } /* failed */

		      dlg.AddPage(&index);

		      // Note that we already have a PCI data object, so
		      // the loop below starts out by formatting it
		      PCIData * data;
		      while(TRUE)
			 { /* scan PCI */
			  CString s;
			  s.Format(_T("%d:%d:%d"), 
			  		pciBuffer.BusNumber,
			  		pciBuffer.SlotData.u.bits.DeviceNumber,
					pciBuffer.SlotData.u.bits.FunctionNumber);
			  // Which one we declare here may depend on
			  // whether or not we choose to support other than
			  // type0 PCI

			  // For right now, "PCIData" supports Type0 only
			  // we may want to derive subclasses in the future
			  // class PCIType0 : public PCIData, etc.

			  data = new PCIData(s);
			  data->data = pciBuffer;
			  dlg.AddPage(data);

			  ZeroMemory(pciBuffer.PciData.DeviceSpecific,
		      		     sizeof(pciBuffer.PciData.DeviceSpecific));

			  result = DeviceIoControl(pci,
		      			           IOCTL_PCISCAN_GET_NEXT,
						   NULL,
						   0,
						   &pciBuffer,
						   sizeof(pciBuffer),
						   &length,
						   NULL);
                          if(!result)
			     break; // stops at end
		          c_Progress.StepIt();
			 } /* scan PCI */
		     } /* do enumeration */
		     state = STATE_CLOSING;
		     break;

	     case STATE_CLOSING: 
		     c_Progress.ShowWindow(SW_HIDE);
		     ::CloseHandle(pci);
		     state = STATE_WATCHING;
		     break;

	     case STATE_WATCHING:
		     ShowWindow(SW_HIDE);
		     dlg.DoModal();
		     ShowWindow(SW_SHOW);
		     UpdateWindow(); // force update
		     state = STATE_STOPPING;
		     break;

	     case STATE_STOPPING:
		     scm = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
		     if(scm == NULL)
			{ /* failed */
			 err = GetLastError();
			 errorstate = state;
			 errormessage = IDS_SCM_OPEN_FAILURE;
			 state = STATE_ERROR;
			 stateRecover = STATE_COMPLETED;
			 continue;
			} /* failed */

		     c_Stopping.SetCheck(BST_CHECKED);

		     if(!StopDriver(scm, PCISCAN))
			{ /* failed */
			 err = GetLastError();
			 errorstate = state;
			 errormessage = IDS_ERROR_STOPPING;
			 state = STATE_ERROR;
			 continue;
			} /* failed */

		     state = STATE_REMOVING;
		     break;
	     case STATE_REMOVING:
		     c_Uninstalling.SetCheck(BST_CHECKED);
		     if(!RemoveDriver(scm, PCISCAN))
			{ /* failed */
			 err = GetLastError();
			 errorstate = state;
			 errormessage = IDS_ERROR_REMOVING;
			 state = STATE_ERROR;
			 continue;
			} /* failed */

		     state = STATE_COMPLETED;
		     break;
	     case STATE_COMPLETED:
		     running = FALSE;
		     break;
	    } /* state */
	} /* run loop */

     if(scm != NULL)
	CloseServiceHandle(scm);

     CString s;
     s.LoadString(IDS_COMPLETED);
     c_Completed.SetWindowText(s);

     updateControls();
     return 0;
    }

BOOL CPCIStartup::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
 	running = FALSE;
 	c_Progress.ShowWindow(SW_HIDE);

	HICON m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	CString strAboutMenu;
	strAboutMenu.LoadString(IDS_ABOUTBOX);

	if (!strAboutMenu.IsEmpty())
	   {
	    pSysMenu->AppendMenu(MF_SEPARATOR);
	    pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
	   }

	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	PostMessage(UWM_RUN);

	CString s;
	s.LoadString(IDS_EXIT);
	c_OK.SetWindowText(s);

	c_Completed.SetWindowText(_T(""));

	updateControls();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

/****************************************************************************
*                            CPCIStartup::OnBrowse
* Result: void
*       
* Effect: 
*       Browses for a file name by bringing up a file dialog.
****************************************************************************/

void CPCIStartup::OnBrowse() 
{
 CString filename;
 c_Driver.GetWindowText(filename);
 if(!browse(filename))
    return;

 c_Driver.SetWindowText(filename);
	
}

void CPCIStartup::OnSysCommand(UINT nID, LPARAM lParam)
    {
     if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
	 CAboutDlg dlgAbout;
	 dlgAbout.DoModal();
	}
     else
	{
	 CDialog::OnSysCommand(nID, lParam);
	}
}


/****************************************************************************
*                            CPCIStartup::OnChange
* Result: void
*       
* Effect: 
*       Changes the file registered for this program to the one specified
*	in the file box
****************************************************************************/

void CPCIStartup::OnChange() 
{
 if(c_Driver.GetWindowTextLength() == 0)
    return; // shouldn't happen

 RegistryString filename(IDS_PCISCAN);
 c_Driver.GetWindowText(filename.value);
 
 filename.store();
}

/****************************************************************************
*                         CPCIStartup::OnChangeDriver
* Result: void
*       
* Effect: 
*       Each time the driver name changes, validates that it is a valid
*	name
****************************************************************************/

void CPCIStartup::OnChangeDriver() 
{
 UINT msg;
     
 updateControls(&msg);	
 if(msg != 0)
    { /* show message */
     CString s;
     s.LoadString(msg);
     c_Status.SetWindowText(s);
    } /* show message */
  else
      c_Status.SetWindowText(_T(""));
}
